package Mongoose::Role::Expander;
$Mongoose::Role::Expander::VERSION = '0.36';
use Moose::Role;

requires 'expand';

=head1 NAME

Mongoose::Role::Expander

=head1 DESCRIPTION

The expander role. No moving parts. Used by the engine. 

=cut 

1;
